#import arcgisscripting
import arcgisscripting
#import dasyLib
gp = arcgisscripting.create(9.3)
gp.OverwriteOutput = True

# required inputs
InFeatures = gp.GetParameterAsText(0) #"C:/Documents and Settings/mgould/temp/inputs_DME/Census/clat_bg.shp"   
InRas = gp.GetParameterAsText(1) #"C:/Documents and Settings/mgould/temp/inputs_DME/Land_Use_Ancillary/clat_4classm"
InField = gp.GetParameterAsText(2) #"AreaKey"
populationClassName = gp.GetParameterAsText(3) #"POP2000"
unhibClass = gp.GetParameterAsText(4)
percent = gp.GetParameterAsText(5)
gp.workspace = gp.GetParameterAsText(6)


#########################stand alone presets for debug #######################
#InFeatures = "C:/Users/mgould/Documents/temp/dasyImputs/inputs_DME/Census/clat_bg.shp"   
#InRas ="C:/Users/mgould/Documents/temp/dasyImputs/inputs_DME/Land_Use_Ancillary/clat_4classm"
#InField = "AreaKey"
#populationClassName = "POP2000"
#popRasterClassName = "POPRASTER"
#unhibClass = 0
#percent = 80
#OutRaster = "C:/Users/mgould/temp/dasy2014Workspace/popraster"
#InRasField = "CLAT_4CLASSM"


workFolder = gp.workspace.replace('\\','/')
OutRaster = workFolder +"/popRaster"
Dasy_Table = workFolder +"/Dasy_Rast"
InRasField = (InRas.split("\\" ))[-1]
                                            
# Set required varibles
popClassList = []  # list to hold rasterized population ID's, field name "POPRASTER"
landUseList = []  # list to hold land use types,example 1-4


# Rasterize the Population feature file (clat_bg.shp)
# Process: FeatureToRaster_conversion
try:
    # Set cell size
    dsc = gp.describe(InRas)
    InCellSize = (dsc.MeanCellHeight + dsc.MeanCellWidth)/ 2
    gp.FeatureToRaster_conversion(InFeatures, InField, OutRaster, InCellSize)
    
except:
    # Print error message if an error occurs
    gp.AddMessage(gp.GetMessages())


# join/combine the landuse file to the new pop raster file. Name new file "Dasy_Rast"
try:
    # Check out Spatial Analyst extension license
    if gp.CheckExtension("spatial") == "Available":
        gp.CheckOutExtension("spatial")
    else:
        raise "LicenseError"
except "LicenseError":
    gp.AddMessage("Spatial Analyst license is unavailable")
except:
    gp.AddMessage(gp.GetMessages(2))

gp.AddMessage(OutRaster)
try:
    # Process: Combine...
    gp.Combine_sa(InRas +";" + OutRaster, Dasy_Table)
except:
    # If an error occurred while running a tool, then print the messages.
    gp.AddMessage("Dasy_Rast Creation failed")
    gp.AddMessage(gp.GetMessages())



### correct Dasy_Rast population raster field  if needed to mach FID field, FIDs start with 0
rows = gp.searchcursor(OutRaster,"","","VALUE",)
row = rows.next()
gp.Addmessage("Checking Dasy Table POPRASTER Field starts at 0") 
if row.VALUE == 1:
  try:
        # Calculate field to a new value
    gp.Addmessage("Correcting Dasy Table POPRASTER Field to start at 0")
    gp.CalculateField_management(Dasy_Table, "POPRASTER", "[POPRASTER] - 1", "VB")
  except:
        # If an error occurs , print out the error message.
    gp.AddMessage("Dasy_Rast correction failed")  

del rows
del row

### Join the popultion table (clat_bg) to Dasy_Rast
try:
    gp.joinfield (Dasy_Table, "POPRASTER", InFeatures, "FID")
except:
    # If an error occurs 
    gp.AddMessage(gp.GetMessages())


# add needed fields to Dasy_Rast
gp.toolbox = "management"
try:
    # Add fields  of type LONG
    gp.addfield (Dasy_Table, "TRCTCOUNT", "long")
    gp.addfield (Dasy_Table, "INHAB", "long")
    gp.addfield (Dasy_Table, "INHAB_CNT", "long")
    gp.addfield (Dasy_Table, "SAM_BLK", "TEXT")
    gp.addfield (Dasy_Table, "INHAB_DENS", "float")
    gp.addfield (Dasy_Table, "POPEST", "float")
    gp.addfield (Dasy_Table, "TOTALFRACT", "float")
    gp.addfield (Dasy_Table, "NEWPOP", "float")
    gp.addfield (Dasy_Table, "NEWDENSITY", "float")
    gp.addfield (Dasy_Table, "METHOD", "TEXT")
except:
    # If an error occurs when running Addfield, print out the error message.
    gp.AddMessage(gp.GetMessages())

# Set the INHAB Field based on user selected unihabted class
gp.Addmessage("Setting the INHAB Field")

#  create an object gdb to hold a dasy_table row
class gdb(object):
    try:
        def __init__(self, rowid=0, value=0, count=0, popraster=0, clat_4classm=0, pop2000= 0 ):
            self.ROWID = rowid
            self.VALUE = value
            self.COUNT = count
            self.POPRASTER = popraster
            self.CLAT_4CLASSM = clat_4classm
            self.POP2000 = pop2000
            self.TRCTCOUNT = 0
            self.INHAB= 0
            self.INHAB_CNT= 0
            self.SAM_BLK= "No"
            self.INHAB_DENS= 0
            self.POPEST= 0
            self.TOTALFRACT= 0
            self.NEWPOP= 0
            self.NEWDENSITY= 0
            self.METHOD = None
            
    except Exception:
        print (traceback.format_exc())
# Create ab array to hold dasy_table data in memeory and load it with data from Dasy_table      
dasy_gdb=[]
rows = gp.searchcursor(Dasy_Table)
row = rows.next()         
while row:
    dasy_gdb.append(gdb(row.Rowid,row.VALUE,row.COUNT,row.POPRASTER,row.getValue(InRasField),row.getValue(populationClassName)))  
    row = rows.next()

del row
del rows

#Set INHAB data member in dasy_gdb based on user selected unihabted class
for entry in dasy_gdb:
    if str(entry.CLAT_4CLASSM) != unhibClass:
      entry.INHAB = 1
    else:
      entry.METHOD =  "Preset to 0"



# Create lists of all unique popRasterClassNames (example : 1-37) and Land uses
gp.Addmessage("Building a table of population raster id's")
for entry in dasy_gdb:
    popClassList.append(entry.POPRASTER)
    landUseList.append(entry.CLAT_4CLASSM)
    
popClassList = set(popClassList)
landUseList = set(landUseList)

# Set the TRACOUNT Field by summing the count for each population raster id
gp.Addmessage("summing the count for each population raster id")

areaCount = 0
for item in popClassList:
  for entry in dasy_gdb:
      if entry.POPRASTER == item:
          areaCount += entry.COUNT
  for entry in dasy_gdb:         
      if entry.POPRASTER == item:
          entry.TRCTCOUNT = areaCount
  areaCount = 0;
  
# Set the INHABCNT Field by summing the count for each inhabited population raster id
gp.Addmessage("summing the inhabited count for each population raster id")
inhabCount = 0
for item in popClassList:
  for entry in dasy_gdb:
      if entry.POPRASTER == item  and entry.INHAB == 1:
          inhabCount += entry.COUNT
  for entry in dasy_gdb:         
      if entry.POPRASTER == item  and entry.INHAB == 1:
          entry.INHAB_CNT = inhabCount
          entry.INHAB_DENS =  float(entry.POP2000) / float(entry.TRCTCOUNT) 
  inhabCount = 0;

# Set the Population estimate for sampled land use classes
## POPEST and METHOD
gp.Addmessage("Setting the Population estimate for sampled land use classes")

 
pctCov = float(percent) / 100

for item in landUseList:
  totPop = 0
  totArea = 0
  repDensity = 0
  for entry in dasy_gdb:   
    if entry.CLAT_4CLASSM == item and entry.COUNT >= pctCov * entry.INHAB_CNT and entry.INHAB_CNT > 0:
      totArea += entry.INHAB_CNT
      totPop += entry.POP2000
      entry.SAM_BLK = "yes"    

  if totArea > 0:
    repDensity = float(totPop)/float(totArea)
  else:
    repDensity = 0
                              
 ## rows with repDensity of 0 will get a 0 for POPEST

  for entry in dasy_gdb:  
    if entry.CLAT_4CLASSM == item :  
      entry.POPEST = repDensity * entry.COUNT
      if repDensity != 0:
        entry.METHOD = "Sampled"

# Set the Population estimate for unsampled land use classes
gp.Addmessage("Setting the Population estimate for unsampled land use classes")

# Create list of all unsampled land use classes (example : 2,3)
unSampledClassList = []

for item in landUseList:
  rCount = 0  
  for entry in dasy_gdb:
    if entry.CLAT_4CLASSM == item and entry.COUNT >= .8 * entry.INHAB_CNT:
      rCount += 1

  if  rCount < 2:
    unSampledClassList.append(item)
    
### Create list of all popRaster data (example: see pRaster class)
class pRaster:
  def __init__(self,ident):
    self.ident = ident
    self.totPop = 0
    self.estPop = 0
    self.inhabArea = 0
    self.noSampleArea = 0
  def addToTotPop(self, v):
    self.totPop += v
  def addToEstPop(self, v):
    self.estPop += v
  def setInhabArea(self, v):
    self.inhabArea = v
  def addToNoSampleArea(self, v):
    self.noSampleArea += v
    
popRasList = {}
for item in popClassList:
  totPopSet = False  
  popRasList[item] = pRaster(item)
  for entry in dasy_gdb:
    if entry.POPRASTER == item: 
      if totPopSet == False:
          popRasList[item].addToTotPop(entry.POP2000) # only need to do this once
          totPopSet = True
      popRasList[item].addToEstPop(entry.POPEST)
      if entry.INHAB_CNT != 0  and popRasList[item].inhabArea == 0:
        popRasList[item].setInhabArea(entry.INHAB_CNT)
      if entry.POPEST == 0 and entry.INHAB == 1:
        popRasList[item].addToNoSampleArea(entry.COUNT)
  


  ##End of  for entry in dasy_gdb:
        
  excessPop = popRasList[item].totPop - popRasList[item].estPop
  excessArea = popRasList[item].noSampleArea
      
  for entry in dasy_gdb:
    if entry.POPRASTER == item and entry.POPEST == 0 and entry.INHAB == 1:
       if excessPop > 0:
            assignPop = (float(entry.COUNT) / float(excessArea)) * excessPop            
            entry.POPEST = assignPop


# Recalculate unsampled class Density
unSamDensitylist = {}
for item in unSampledClassList:
  totPopEst = 0
  totArea = 0
  for entry in dasy_gdb:
    if entry.CLAT_4CLASSM == item :
      totPopEst += entry.POPEST
      totArea += entry.COUNT

  pNewDensity =  totPopEst / totArea
  unSamDensitylist[item] = pNewDensity
      
         
# Recalculate unsampled class POPEST
for item in unSampledClassList:
  for entry in dasy_gdb:
    if entry.CLAT_4CLASSM == item :
          entry.POPEST = unSamDensitylist[item] * entry.COUNT
          entry.METHOD = "Areal Weighted"


### For each popClassList item iterate through the rows (gdb array) and sum POPESTs, update TRACTFRAC,etc
gp.Addmessage("Performing final calculations")

for item in popClassList:
  estCount = 0
  for entry in dasy_gdb:    
    if entry.POPRASTER == item:
      estCount += entry.POPEST
  for entry in dasy_gdb:
    if entry.POPRASTER == item:
      if estCount > 0: 
        entry.TOTALFRACT = entry.POPEST / estCount 
       
    ###  Calculate and update
    ###    NEWPOP (TOTALFRACT * POP2000) and  
    ###    NEWDENSITY (NEWPOP / COUNT.
              
       

  for entry in dasy_gdb:    
    if entry.POPRASTER == item:
          entry.NEWPOP = entry.TOTALFRACT * entry.POP2000
          if entry.COUNT > 0: 
            entry.NEWDENSITY = entry.NEWPOP / entry.COUNT
          else:
            entry.NEWDENSITY = 0




## put dasy_gdb data into Dasy_table
i=0
rows = gp.UpdateCursor(Dasy_Table)
row = rows.next()
while row:
    row.NEWDENSITY = dasy_gdb[i].NEWDENSITY
    row.INHAB = dasy_gdb[i].INHAB
    row.METHOD = dasy_gdb[i].METHOD
    row.TRCTCOUNT = dasy_gdb[i].TRCTCOUNT
    row.INHAB_CNT = dasy_gdb[i].INHAB_CNT
    row.SAM_BLK = dasy_gdb[i].SAM_BLK
    row.INHAB_DENS = dasy_gdb[i].INHAB_DENS
    row.POPEST = dasy_gdb[i].POPEST
    row.METHOD = dasy_gdb[i].METHOD
    row.TOTALFRACT = dasy_gdb[i].TOTALFRACT
    row.NEWPOP = dasy_gdb[i].NEWPOP
    row.NEWDENSITY = dasy_gdb[i].NEWDENSITY
    i = i+1
    rows.UpdateRow(row)
    row = rows.next()

del row
del rows


